function[lambda, omega, Q] = lambda_omega_Q(u, v, w, zz, z_index,...
    surf_elev, deta_dx_2D, deta_dy_2D)

% Calculating velocity derivatives
[du_dx, du_dy, du_dz] = vel_derivatives(u, zz, z_index,...
    surf_elev, deta_dx_2D, deta_dy_2D);
[dv_dx, dv_dy, dv_dz] = vel_derivatives(v, zz, z_index,...
    surf_elev, deta_dx_2D, deta_dy_2D);
[dw_dx, dw_dy, dw_dz] = vel_derivatives(w, zz, z_index,...
    surf_elev, deta_dx_2D, deta_dy_2D);

% Initializeing the matrices. For each point in the grid we have a 3x3
% matrix for the strain and rotaion stresses. Giving these 5 dimensions
S = zeros(256,256,length(z_index),3,3);
O = S;

% Setting the values for the stress tensor at each point
S(:,:,:,1,1) = 0.5*(du_dx + du_dx);
S(:,:,:,1,2) = 0.5*(du_dy + dv_dx);
S(:,:,:,1,3) = 0.5*(du_dz + dw_dx);
S(:,:,:,2,1) = 0.5*(dv_dx + du_dy);
S(:,:,:,2,2) = 0.5*(dv_dy + dv_dy);
S(:,:,:,2,3) = 0.5*(dv_dz + dw_dy);
S(:,:,:,3,1) = 0.5*(dw_dx + du_dz);
S(:,:,:,3,2) = 0.5*(dw_dy + dv_dz);
S(:,:,:,3,3) = 0.5*(dw_dz + dw_dz);

% Setting the values for the rotation tensor at each point
O(:,:,:,1,1) = 0.5*(du_dx - du_dx);
O(:,:,:,1,2) = 0.5*(du_dy - dv_dx);
O(:,:,:,1,3) = 0.5*(du_dz - dw_dx);
O(:,:,:,2,1) = 0.5*(dv_dx - du_dy);
O(:,:,:,2,2) = 0.5*(dv_dy - dv_dy);
O(:,:,:,2,3) = 0.5*(dv_dz - dw_dy);
O(:,:,:,3,1) = 0.5*(dw_dx - du_dz);
O(:,:,:,3,2) = 0.5*(dw_dy - dv_dz);
O(:,:,:,3,3) = 0.5*(dw_dz - dw_dz);

% Initializing the eigenvalues matrix
lambda    = zeros(256,256,length(z_index));   % Median eigenvalues matrix
omega     = zeros(256,256,length(z_index),3); % Voticity matrix
Q = lambda;                   % Sum og eigenvalues matrix

% Retrieving second largest eigenvalue (lambda_2) and Q matrix values
for i = 1:256
   for j = 1:256
      for k = z_index
         % Temporary index to make the logic fit
         if length(z_index)==1
            tmp_ind = 1;
         else
            tmp_ind = z_index(k);
         end
         S_2D = reshape(S(i,j,tmp_ind,:,:),[3 3]);
         O_2D = reshape(O(i,j,tmp_ind,:,:),[3 3]);
         tmp = eig(S_2D^2 + O_2D^2);
         lambda(i,j,tmp_ind) = median(tmp);
         omega(i,j,tmp_ind,:)=[dw_dy(i,j,tmp_ind) - dv_dz(i,j,tmp_ind),...
                               du_dz(i,j,tmp_ind) - dw_dx(i,j,tmp_ind),...
                               dv_dx(i,j,tmp_ind) - du_dy(i,j,tmp_ind)];
         Q(i,j,tmp_ind) = -0.5*sum(tmp);
      end
   end
end

end