function [surf_upwelling, surf_downwelling ]= ...
    surface_wellings_locations(surf_div)

% Variable interval space averaging half square length
W = 0.22;

% Maximun indexing for the grid. Since the whole surface is 256 by 256,
% and half square length for the variable interval cannot excede the
% domain
W_ind = ceil(W*256/(2*pi));

% Differential area used for integration
dA = (2*pi/255).^2;

% Surface divergence root mean square
surf_div_rms = sqrt(sum(surf_div.^2, 'all')/(256^2));
surf_div_pos = surf_div.*(surf_div>0);
surf_div_neg = surf_div.*(surf_div<0);

% Initializing some variables
surf_div_mean_pos = zeros(256);
surf_div_mean_neg = zeros(256);
surf_div_var_pos  = zeros(256);
surf_div_var_neg  = zeros(256);

% Variabel interval space-averaging for surface divergence
for i = 1:256
    for j = 1:256
        x_int = max(i-W_ind,1):min(i+W_ind,256);
        y_int = max(j-W_ind,1):min(j+W_ind,256);
        % Variabel interval space-averaging quantity
        surf_div_mean_pos(i,j) = dA/(4*W^2).*...
            (sum(surf_div_pos(x_int,y_int), 'all'));
        surf_div_mean_neg(i,j) = dA/(4*W^2).*...
            (sum(surf_div_neg(x_int,y_int), 'all'));
        % Localized varience for the surface divergence
        surf_div_var_pos(i,j) = surf_div_pos(i,j).^2 - surf_div_mean_pos(i,j).^2;
        surf_div_var_neg(i,j) = surf_div_neg(i,j).^2 - surf_div_mean_pos(i,j).^2;
    end
end

% Criterion for a up or downwelling is that the surface local maximum
% variance is 5 times larger than the surface divergence rms value.
surf_upwelling   = surf_div_var_pos.*(surf_div_var_pos>5*surf_div_rms.^2);
surf_downwelling = surf_div_var_neg.*(surf_div_var_neg>5*surf_div_rms.^2);

end
