function [u_rms, taylor_scale] = taylor_length_scale()
% This is a funtion to fint the taylor microscale for the turbulence
% structures in the data given by Gou and Shen near the surface. It is
% based on function (36) & (38) from the 2009 paper from Gou and Shen "On
% the generation and measurement of waves and turbulence in simulations of
% the free surface turbulence". In essence we calculate the microscale in
% the bulk region using the forcing used and the Reynolds number. The use
% the conversion funtion in (38) to find the length scale in the free
% region above.

% Forcing term used in the bulk region (the same as the one used in the Gou
% and Shen article from 2010 "INteraction of a deformable free surface with
% statistically steady homogeneous turbulence". It is the same as the one
% in this simulation, but for the different viscosity giving a different
% Reynolds number.
a_0 = 0.1;

% Reynolds number for this simulation
Re = 2500;

% Constant used in the eq 38 in Gou & Shen 2009, for a damping region of
% pi/2 the values for these are listed in table 2 in the same article
C_l = -1.0124;
D_l = 0.2085;
C_u = 4.3016;
D_u = -0.9681;

% B_l = 0.4245;
% B_u = 0.3114;

% This is the distace from the center og the domain where we are tying to
% find the Taylor length scale
z_c = 3*pi/2 + pi/2 + 0.5*pi/2;

% Taylor length scale in the bulk region, see eq 36 in Gou & Shen 2009
bulk_scale = 2.24/sqrt(a_0*Re);
u_rms_bulk = 0.57*a_0*2*pi;

% The taylor scale wery close to the surface
taylor_scale = bulk_scale * exp(C_l + D_l*z_c);
u_rms = sqrt(u_rms_bulk^2 * exp(C_u + D_u*z_c));

% taylor_scale = bulk_scale * B_l * z_c;
% u_rms = u_rms_bulk * B_u / z_c;

end