%% Total inspection
clear

% Referance scales and flow variables
Re    = 2500;                  % Reynold's Number
Fr    = sqrt(0.01);            % Frode's Number
l_ref = 2*pi;                  % Reference length
H_bar = 5*pi;                  % Domain height
a_0   = 0.1;                   % Forcing parameter
u     = a_0*l_ref;             % Referance velocity
nu    = 1/Re;                  % Kinematic viscosity

% Loading surface elevation and timesteps
[eta0, timestamps] = load_surface_data();

% Square half side lenght
ds = 0.1;

% Cutoff value for veortex inspection
cutoff_lambda = 2;

% Wavelt scale index for inspection (i.e. the index in the scale vector for
% the scale of intereset)
wlt_scale = 1;

% Taylor microscale very close to the surface
[u_rms, taylor_scale] = taylor_length_scale();
% Taylor Reynlods number
Re_lambda = u_rms*taylor_scale/nu;

% Initializing vortex strength vector
vortex_strength = [];

% Testing variable
vortex_strength_2 = [];

% Initializing vortex strength vector
ds_ind = floor(ds*256/(2*pi))*2 + 1;
vortex_wlt_max = [];
vortex_wlt_int = [];

% Testing variable
vortex_wlt_max_2 = [];
vortex_wlt_int_2 = [];

% Differential area used for integration
dA = (2*pi/255).^2;

% Loading z coordinates
[~,~,~,~,zz] = load_near_surf_velocities(1);

% Important to note that zz is 1 at the suirface and decreas as we go 
% further down in the domain
% Surface mesh
x_range = linspace(0, l_ref, 256);
y_range = linspace(0, l_ref, 256);
[x_2d, y_2d] = meshgrid(x_range, y_range);
[x_3d, y_3d, z_3d] = meshgrid(x_range, y_range, zz);
[x_ver, z_ver] = meshgrid(x_range, zz);
% [x_int, y_int, z_int] = meshgrid(x_range, y_range, zw);

% Index of horizontal slice for inspection
% insp_ind = 1:length(zz);
insp_ind = 1;
z_ind = 1;

% Initializing vortex strength variables
true_vortex_strength = zeros(1,length(timestamps));
vortex_wlt_strength  = zeros(1,length(timestamps));
vortex_elev_strength = zeros(1,length(timestamps));


%% Velocity field comparison at different timesteps

for data_index = 5
% Data index for the timestamp we are looking at

% Loading subsurface velocities (u,v,w) and the presure (p)
[v,u,w,p,zz] = load_near_surf_velocities(data_index);

% Surface elevation at this time index
surf_elev = reshape(eta0(data_index,:,:),[256 256]);


%% Finding the derivatives

% Differentiating the surface elevation
% Time step for differentiation
dt = (timestamps(2) - timestamps(1));

% Grid variables in the x ang y directions
xhi = linspace(0,l_ref,256);
psi = linspace(0,l_ref,256);
dx  = xhi(2)-xhi(1);

% The relevant derivatives
[deta_dt, ~, ~] = gradient(eta0, dt, dx, dx);

% Surface gradients at the investigated timestamp
[deta_dx, deta_dy] = gradient(surf_elev, xhi, psi);

% z_index = 1:length(zz);

% Calculating velocity derivatives 
[du_dx, ~, ~] = vel_derivatives(u, zz, insp_ind,...
    surf_elev, deta_dx, deta_dy);
[~, dv_dy, ~] = vel_derivatives(v, zz, insp_ind,...
    surf_elev, deta_dx, deta_dy);

% The rest if needed
% [du_dx, du_dy, du_dz] = vel_derivatives(u, zz, insp_ind,...
%     surf_elev, deta_dx, deta_dy);
% [dv_dx, dv_dy, dv_dz] = vel_derivatives(v, zz, insp_ind,...
%     surf_elev, deta_dx, deta_dy);
% [dw_dx, dw_dy, dw_dz] = vel_derivatives(w, zz, insp_ind,...
%     surf_elev, deta_dx, deta_dy);
% [dp_dx, dp_dy, dp_dz] = vel_derivatives(p, zz, insp_ind,...
%     surf_elev, deta_dx, deta_dy);

% Surface gradient and surface curvature
% Finding the second order derivatives
[deta_dxx, deta_dxy] = gradient(deta_dx, xhi, psi);
[~, deta_dyy] = gradient(deta_dy, xhi, psi);

% Surface gaussian curvature, the product of the principal curvatures
surf_gaus_curv = (deta_dxx.*deta_dyy-deta_dxy.^2)./...
    (1 + deta_dx.^2 + deta_dy.^2).^2;

% Surface mean curvature, the mean of the principal curvatures
surf_mean_curv = ((1+deta_dy.^2).*deta_dxx + (1+deta_dx.^2).*deta_dyy ...
    - 2*deta_dx.*deta_dy.*deta_dxy)./(1 + deta_dx.^2 + deta_dy.^2).^(3/2);

% Inicializing principal curvatures
surf_prin_curv_1 = zeros(256,256);
surf_prin_curv_2 = zeros(256,256);

% Setting the principal curvatures
for i = 1:256
    for j = 1:256
        surf_II_form = [deta_dxx(i,j) deta_dxy(i,j);...
            deta_dxy(i,j) deta_dyy(i,j)];
        prin_curv = eig(surf_II_form);
        surf_prin_curv_1(i,j) = max(prin_curv);
        surf_prin_curv_2(i,j) = min(prin_curv);
    end
end

% Mean surface elevation
surf_mean = mean(surf_elev,'all');

%% Finding vortices

% Getting the middel eigenvalue at each point (lambda), the vorticity inn
% all trhee directions at each point (omega) and the sum og eignevalues Q
[lambda, omega, Q] = lambda_omega_Q(u, v, w, zz, insp_ind,...
    surf_elev, deta_dx, deta_dy);

% Checking for votices
vortex_pos = zeros(256,256); % Initializing variable
vortex_omega = zeros(256,256);

% When the second eigenvalue (lambda_2) is negative we have a vortex core
% When it is positive we don't, leaving that at 0
% Bathtub vortices will mainly have vorticity in the z directiions
if length(insp_ind) == 1
    vortex_pos = lambda(:,:,1).*(lambda(:,:,1)<0);
    
    % z-direction vorticity for bathtub vortices
    omega_z = reshape(omega(:,:,1,3),[256 256]);
    vortex_omega(lambda<-cutoff_lambda) = omega_z(lambda<-cutoff_lambda);
else
    vortex_pos = lambda(:,:,z_ind).*(lambda(:,:,z_ind)<0);
    
    % z-direction vorticity for bathtub vortices
    omega_z = reshape(omega(:,:,z_ind,3),[256 256]);
    vortex_omega(lambda(:,:,z_ind)<-cutoff_lambda) = ...
        omega_z(lambda(:,:,z_ind)<-cutoff_lambda);
end

% Calculating enstrophy
enstrophy = sum(omega.^2,4);

% Defining x and y coordinates for som bathtub vortices 
[pos_x_vec, pos_y_vec] = bathtub_pos_vec(data_index);
pos_x_vec_2 = (2*pi-2*ds)*0.99*rand(1,10)+ds;
pos_y_vec_2 = (2*pi-2*ds)*0.99*rand(1,10)+ds;

% Calculating vortex strengths
if length(insp_ind) == 1
    for i = 1:length(pos_x_vec)
        vortex_strength(end+1) = vortex_strength_by_int(vortex_pos,...
            enstrophy, pos_x_vec(i), pos_y_vec(i));
        vortex_strength_2(end+1) = vortex_strength_by_int(vortex_pos,...
            enstrophy, pos_x_vec_2(i), pos_y_vec_2(i));
    end
else
    for i = 1:length(pos_x_vec)
        tmp_strength = [];
        tmp_strength_2 = [];
        for ind = z_ind
            tmp_strength(ind) = vortex_strength_by_int(vortex_pos,...
                enstrophy(:,:,ind), pos_x_vec(i), pos_y_vec(i));
            tmp_strength_2(ind) = vortex_strength_by_int(vortex_pos,...
                enstrophy(:,:,ind), pos_x_vec_2(i), pos_y_vec_2(i));
        end
        vortex_strength(end+1) = sum(tmp_strength);
        vortex_strength_2(end+1) = sum(tmp_strength_2);
    end
end

true_vortex_strength(data_index) = ...
    sum(enstrophy.*(vortex_pos<-1.5), 'all')*dA;
vortex_elev_strength(data_index) = ...
    sum(surf_elev.*(vortex_pos<-1.5), 'all')*dA;

%% Finding up and downwellings

% Surface divergence
surf_div = du_dx + dv_dy;

% Probable location of up and downwellings
[surf_upwelling, surf_downwelling] = surface_wellings_locations(surf_div);


%% Isotropic Wavelet transformation
% Wavelet name
wname_iso = "mexh";
wname_iso_full = "Mexican hat";

% Scale range for transform
scale_range = logspace(0, log10(2^3), 25);

% Wavelet transformation of the surface
cwt_elev_iso = cwtft2(surf_elev, 'wavelet', wname_iso, ...
    'scales', scale_range);

% Matrix the coefficients at each frequency
wlt_elev_iso = reshape(cwt_elev_iso.cfs(:,:,1,:), ...
    [256 256 length(scale_range)]);
angles_coeff_lev = angle(wlt_elev_iso); % Phase angle of the coefficients

% Calculating vortex strengths
for i = 1:length(pos_x_vec)
    [vortex_wlt_max(end+1), vortex_wlt_int(end+1)] = ...
        vortex_wlt_trans(vortex_pos, wlt_elev_iso, ...
        pos_x_vec(i), pos_y_vec(i));
    [vortex_wlt_max_2(end+1), vortex_wlt_int_2(end+1)] = ...
        vortex_wlt_trans(vortex_pos, wlt_elev_iso, ...
        pos_x_vec_2(i), pos_y_vec_2(i));
end

[wlt_iso, wlt_iso_len_scale] = ...
    wavelet_length_scale(scale_range(wlt_scale),surf_elev, wname_iso);

% Vortex position circles from wavelet transformation and vortex strength 
% based on the transform of the elevation
% [vortex_wlt_strength(data_index), vor_circles, vor_radii] = ...
%     wlt_surface_vortices(wlt_elev_iso(:,:,1), surf_mean_curv);

% Calculating vortex strength based on the transform of the elevation
% vortex_wlt_strength(data_index) = vortex_wavelet_strength(vor_circles,...
%     vor_radii, wlt_elev_iso(:,:,1));

%% Nonisotropic wavelet transformation

wname_mexh_non = {"mexh", {2,1,0.2}}; % Mexican hat wvaelet 2D
wname_morl_non = {"morl", {6,1,1}}; % Morlet wavelet 2D

% Scale range for transform
scale_range = logspace(0, log10(2^3), 25);

% Angles used in the transform
angles = linspace(-pi/2, pi/2, 13);
% angles = [0 15 30 45 60 75 90 105]/180*pi;

cwt_elev_non = cwtft2(surf_elev, 'wavelet', wname_mexh_non, ...
    'angles', angles, 'scales', scale_range);

wlt_elev_non = reshape(cwt_elev_non.cfs, [256 256 ...
    length(scale_range) length(angles)]);

[wlt_non, wlt_non_len_scale] = ...
    wavelet_length_scale(scale_range(wlt_scale),surf_elev, wname_mexh_non);

wlt_scars = zeros(256, 256);

for i = 1:length(angles)
    wlt_scars = wlt_scars + (real(wlt_elev_non(:,:,wlt_scale,i))<-1.1*10^-4);
end


%% Stefan's idea: comparison with surface curvature

x_resize = 32;
xhi_resize = linspace(0,2*pi,x_resize);
surf_elev_resize = imresize(surf_elev, [x_resize x_resize]);

[deta_dx_resize, deta_dy_resize] = ...
    gradient(surf_elev_resize, xhi_resize, xhi_resize);

[deta_dxx_resize, deta_dxy_resize] = ...
    gradient(deta_dx_resize, xhi_resize, xhi_resize);
[~, deta_dyy_resize] = gradient(deta_dy_resize, xhi_resize, xhi_resize);

surf_mean_curv_resize = ((1+deta_dy_resize.^2).*deta_dxx_resize ...
    + (1+deta_dx_resize.^2).*deta_dyy_resize ...
    - 2*deta_dx_resize.*deta_dy_resize.*deta_dxy_resize)./...
    (1 + deta_dx_resize.^2 + deta_dy_resize.^2).^(3/2);

surf_mean_curv_resize_2 = imresize(surf_mean_curv, [x_resize x_resize]);

cwt_elev_iso_resize = cwtft2(surf_elev_resize, 'wavelet', wname_iso, ...
    'scales', 1 );

wlt_elev_iso_resize = reshape(cwt_elev_iso_resize.cfs(:,:,1,:), ...
    [x_resize x_resize]);

%% Timestep finished

fprintf("Timestep %5.2f done, nr %3d of %d \n",...
    timestamps(data_index), data_index, length(timestamps))

end


%% Ploting

if data_index == 160

    figure(1), clf
    plot(timestamps, true_vortex_strength./vortex_wlt_strength)
    xlim([timestamps(1) timestamps(end)])
    title("Comparison of vortex strength at each timestep")

else
    figure(1), clf
%     subplot(1,2,1)
    pcolor(x_2d, y_2d, surf_elev)
    hold on
%     contour(x_2d, y_2d, -vortex_pos*0.5*10^-4, [1.5 1.5]*0.5*10^-4,...
%         'r', 'linewidth', 1.5)
    contour(x_2d, y_2d, surf_downwelling*10^-4, [0.1 0.1]*10^-4,...
        'g', 'linewidth', 1)
    contour(x_2d, y_2d, surf_upwelling*10^-4, [0.1 0.1]*10^-4,...
        'r', 'linewidth', 1)
%     viscircles([1,0.9],0.7, 'color', 'w');
    shading flat
    colorbar
    xlim([0,2*pi])
    ylim([0,2*pi])
    xlabel("x axis")
    ylabel("y axis")
    legend("Surface elevation", "Downwellings", "Upwellings")
%     title("Surface elevation at time = " + timestamps(data_index))
    hold off
    
    figure(2), clf
    pcolor(x_2d, y_2d, wlt_elev_iso(:,:,wlt_scale))
    hold on
%     contour(x_2d, y_2d, -vortex_pos*0.5*10^-4, [1.5 1.5]*0.5*10^-4,...
%         'r', 'linewidth', 1)
%     contour(x_2d, y_2d, surf_downwelling*10^-4, [0.1 0.1]*10^-4,...
%         'g', 'linewidth', 1)
%     contour(x_2d, y_2d, surf_upwelling*10^-4, [0.1 0.1]*10^-4,...
%         'r', 'linewidth', 1)
    shading flat
    colorbar
    xlim([0,2*pi])
    ylim([0,2*pi])
    xlabel("x axis")
    ylabel("y axis")
%     legend("Wavelet transform", "Downwellings", "Upwellings")
%     title("Wavelet transform of surface elevation at scale = "...
%         + scale_range(1))
    hold off

    wlt_angle = 7;

    figure(3), clf
    pcolor(x_2d, y_2d, real(wlt_elev_non(:,:,wlt_scale, wlt_angle)))
    shading flat
    colorbar
    xlim([0,2*pi])
    ylim([0,2*pi])
    xlabel("x axis")
    ylabel("y axis")
    title("Anisotropic mexh transform with angle  = " ...
        + angles(wlt_angle)/pi*180)
    hold off

    figure(4), clf
    % contour_levels = [cutoff_lambda cutoff_lambda];
%     contourf(x_2d, y_2d, wlt_elev_iso(:,:,wlt_scale), [0.4 0.4]*10^-4)
    pcolor(x_2d, y_2d, surf_mean_curv)
    hold on
%     contourf(x_2d, y_2d, real(wlt_elev_non(:,:,wlt_scale, 1)), ...
%         [1.2 1.2]*10^-4)
%     contour(x_2d, y_2d, -vortex_pos*10^-4, [0.7 0.7]*10^-4,...
%         'r', 'linewidth', 1)
%     viscircles(vor_circles,vor_radii);
    shading flat
    colorbar
    xlim([0,2*pi])
    ylim([0,2*pi])
    xlabel("x axis")
    ylabel("y axis")
    % legend("Wavelet transform", "Vorteces")
%     title("Wavelet transform positive values and attached vortices")
    hold off

    figure(5), clf
    % pcolor(x_2d, y_2d, wlt_coeff(:,:,1))
    contourf(x_2d, y_2d, -wlt_elev_iso(:,:,wlt_scale), [0.8 0.8]*10^-4)
%     contourf(x_2d, y_2d, real(-wlt_elev_non(:,:,wlt_scale, wlt_angle)),...
%         [1 1]*10^-4)
%     contourf(x_2d, y_2d, wlt_scars, [1 1]*10^-4 )
%     contourf(x_2d, y_2d, -surf_mean_curv, [0.03 0.03])
    hold on
    % contourf(x_2d, y_2d, -wlt_coeff(:,:,1), [1 1]*10^-4)
    contour(x_2d, y_2d, surf_downwelling*10^-4, [0.1 0.1]*10^-4,...
        'g', 'linewidth', 1)
    contour(x_2d, y_2d, surf_upwelling*10^-4, [0.1 0.1]*10^-4,...
        'r' ,'linewidth', 1)
    % contourf(x_2d, y_2d, 10*wlt_coeff(:,:,1), [1 1]*10^-3)
    shading flat
    % colorbar
    xlim([0,2*pi])
    ylim([0,2*pi])
    xlabel("x axis")
    ylabel("y axis")
%     legend("Wavelet transform", "Downwellings", "Upwellings")
%     title("Wavelet transform negative and posisive values comparison")
    hold off
    
    figure(6), clf
    pcolor(x_2d, y_2d, surf_mean_curv)
    hold on
    contour(x_2d, y_2d, surf_downwelling*10^-4, [0.1 0.1]*10^-4,...
        'g', 'linewidth', 1)
    contour(x_2d, y_2d, surf_upwelling*10^-4, [0.1 0.1]*10^-4,...
        'r', 'linewidth', 1)
    shading flat
    colorbar
    xlim([0,2*pi])
    ylim([0,2*pi])
    xlabel("x axis")
    ylabel("y axis")
    title("Surface mean curvature")
    hold off
    
    
    figure(7), clf
%     subplot(1,2,1)
    pcolor(surf_mean_curv_resize)
    shading flat
%     title("Mean curvature og the resized surface")
    figure(8), clf
%     subplot(1,2,2)
    pcolor(wlt_elev_iso(:,:,25))
    shading flat
%     title("Wavelet transform at scale = " + scale_range(25))
%     
%     figure(8), clf
%     plot(vortex_strength./vortex_wlt_max)
%     xlim([1,length(vortex_strength)])
%     
%     figure(9)
%     plot(vortex_strength_2./vortex_wlt_max_2)
%     xlim([1,length(vortex_strength_2)])
end


