function[du_dx, du_dy, du_dz] = vel_derivatives(u, zz, z_index,...
    surf_elev, deta_dx_2D, deta_dy_2D)
    
    % Function for finding all derivativatices at a certain elevation.
    % Saves computational saving space since the derivatives are not
    % calculated everywhere.
    
    H_bar = 5*pi; % initializing domain height
    l_ref = 2*pi; % reference length scale
    
    % Grid variables in the x ang y directions
    xhi = linspace(0, l_ref, 256);
    psi = linspace(0, l_ref, 256);
    
    % Differentioation with the grid tranformed variables
    [du_dxhi, du_dpsi, du_dzeta] = gradient(u,xhi,psi,zz);
    
    % Temporary variable for matrix calculations
    tmp = zeros(256, 256, length(zz));
    
    % Correction to the x derivative
    for i = z_index
        tmp(:,:,i) = zz(i)*deta_dx_2D; end
    du_dx = du_dxhi(:,:,z_index) - ...
        tmp(:,:,z_index).*du_dzeta(:,:,z_index)./(surf_elev + H_bar);
    
    % Correction to the y derivative
    for i = z_index
        tmp(:,:,i) = zz(i)*deta_dy_2D; end
    du_dy = du_dpsi(:,:,z_index) - ...
        tmp(:,:,z_index).*du_dzeta(:,:,z_index)./(surf_elev + H_bar);
    
    % Correction to the z derivative
    du_dz = du_dzeta(:,:,z_index)./(surf_elev + H_bar);
end