function v_strength = vortex_strength_by_int(vortex_pos, enstrophy, pos_x, pos_y)

% Function to calculate the strenght of a vortex at a ceratin level near
% the surface by integrationg the enstrophy (magnitude of the vorticity
% squared, or omega^2) 

% Making a square around the surface structure for inspection
ds = 0.09; % Square half side lenght
% Mesh points number for sqaure half side length
ds_ind = floor(ds*256/(2*pi));

% Center indices for the structure
x_ind = floor(pos_y/(2*pi)*256); % x index for structure
y_ind = floor(pos_x/(2*pi)*256); % y index for structure

% Differential area used for integration
% Here 2*pi/255 is the distance between two grid points in either the x or 
% y direction since the grid is structured
dA = (2*pi/255).^2;

% Extracting index on the grid for the relevatn structue
% x interval for vortex structure
x_interval = [x_ind - ds_ind, x_ind + ds_ind];
% y interval for vortex structure
y_interval = [y_ind - ds_ind, y_ind + ds_ind];

% focusing on points close to the vortex core
vortex_pos = reshape(vortex_pos(x_interval(1):x_interval(2)...
    , y_interval(1):y_interval(2)), [2*ds_ind+1 2*ds_ind+1]);
enstrophy  = reshape(enstrophy(x_interval(1):x_interval(2)...
    , y_interval(1):y_interval(2)), [2*ds_ind+1 2*ds_ind+1]);

% Filtering out the enstrophy to only the areas around the vortex core,
% defined by when vortex_pos is bigger than 0
enstrophy_corr = enstrophy.*(vortex_pos<0);

% Integrating enstrophy to find estimate for vortex strength
v_strength = sum(enstrophy_corr,'all').*dA;

% surf(enstrophy_corr)
% shading flat
% colorbar
end