function vor_wlt_strength = vortex_wavelet_strength(...
    circles, radii, wlt_elev_iso)

% This function estimates the total vortex strength at single timestep by
% integrating the wavelet transform of the surface elevation, but only in
% the areas where the vortices were identified in the tranform. This was
% done by finding circular shapes. So integration is done in areas centered
% in these circles and with width a little bit larger that the diameter, to
% ensue the whole structure is accounted for.

% Mesh for the original domain
x_orig = linspace(0,2*pi,256);
y_orig = x_orig;
[x_orig,y_orig] = meshgrid(x_orig,y_orig);

% Interpolation mesh
int_size = 2000;
x = linspace(0, 2*pi, int_size);
y = x;
[x,y] = meshgrid(x,y);

% Interpolating surface elevation for more points
wlt_elev_int = interp2(x_orig,y_orig,wlt_elev_iso,x,y);
wlt_elev_pos = wlt_elev_int.*(wlt_elev_int>0);

% Variable chanco for circle centers and radii
circles = circles./(2*pi/int_size);
radii = radii./(2*pi/int_size);

% Preallocating total strength variable
vor_wlt_strength = 0;

% Differential area used in integration
dA = (2*pi/int_size).^2;

% ds = round(max(vor_radii)*2);

for i = 1:length(circles)
    % Side length for integration domain for each vortex
    ds = round(radii(i)*2.5);
    
    % Indeces for circle centers
    x_ind = round(circles(i,2));
    y_ind = round(circles(i,1));
    
    % Integration area indeces
    x_interval = [max(x_ind - ds, 1), min(x_ind + ds, int_size)];
    y_interval = [max(y_ind - ds, 1), min(y_ind + ds, int_size)];
    
    % Preallocating partition of the total area
    par_wlt_elev = wlt_elev_pos(x_interval(1):x_interval(2)...
        ,y_interval(1):y_interval(2));
    for x = 1:length(x_interval)
        for y = 1:length(y_interval)
            if sqrt((x_interval(x)-x_ind)^2 + (y_interval(y)-y_ind)^2) > ds
                par_wlt_elev(x,y) = 0;
            end
        end
    end
    
    vor_wlt_strength = vor_wlt_strength + dA*sum(par_wlt_elev,'all');

end

end