function [v_wlt_max, v_wlt_int] = vortex_wlt_trans(vortex_pos,...
    wlt_coeff, pos_x, pos_y)

% Function to calculate the strenght of a vortex at a ceratin level near
% the surface by integrationg the enstrophy (magnitude of the vorticity
% squared, or omega^2) 

% pos_x and pos_y are coordinates for a surface dimple. Due to the finite 
% domain size these values have limits. Minimum for both is ds, and 
% maximum is 2*pi-ds since the domain is between 0 and 2*pi

% Making a square around the surface structure for inspection
ds = 0.09; % Square half side lenght
% Mesh points number for sqaure half side length
ds_ind = floor(ds*256/(2*pi));

% The scale chosen is defined by this index. So 1 is the smallest scale,
% and so on till the largest (which depends on the number of scales used in
% the tranformation in the main code
scale_index = 2;

% Center indices for the structure
x_ind = floor(pos_y/(2*pi)*256); % x index for structure
y_ind = floor(pos_x/(2*pi)*256); % y index for structure

% Differential area used for integration. Here 2*pi/255 is the distance
% between two grid points in either the x or y direction since the grid  
% is structured
dA = (2*pi/255).^2;

% Extracting index on the grid for the relevatn structue
% x interval for vortex structure
x_interval = [x_ind - ds_ind, x_ind + ds_ind];
% y interval for vortex structure
y_interval = [y_ind - ds_ind, y_ind + ds_ind];

% focusing on points close to the vortex core
vortex_pos = reshape(vortex_pos(x_interval(1):x_interval(2)...
    , y_interval(1):y_interval(2)), [2*ds_ind+1 2*ds_ind+1]);
wlt_coeff  = reshape(wlt_coeff(x_interval(1):x_interval(2)...
    , y_interval(1):y_interval(2),scale_index), [2*ds_ind+1 2*ds_ind+1]);

% Filtering out the enstrophy to only the areas around the vortex core,
% defined by when vortex_pos is bigger than 0
% v_transform = wlt_coeff.*(vortex_pos<0);
v_transform = wlt_coeff.*(wlt_coeff>0);

% Integrating enstrophy to find estimate for vortex strength
v_wlt_int = sum(v_transform,'all').*dA;

v_wlt_max = max(v_transform,[],'all');

% surf(v_transform)
% shading flat
% colorbar
end