function [vor_wlt_strength, vor_circles, vor_radii] = ...
    wlt_surface_vortices(wlt_elev_iso, surf_mean_curv)

% This function tries to isolate the attached vortices at the surface by
% using the wavelet tranform og the surface elevation. It uses Hough circle
% transfomrs to do so.

% Mesh for the original domain
x_orig = linspace(0,2*pi,256);
y_orig = x_orig;
[x_orig,y_orig] = meshgrid(x_orig,y_orig);

% Interpolation mesh
int_size = 2000;
x = linspace(0, 2*pi, int_size);
y = x;
[x,y] = meshgrid(x,y);


% Threshold for the contour
cont_thresh = 0.5*10^-4;

% Interpolating surface elevation for more points to use in later
% algorithms
wlt_elev_int = interp2(x_orig, y_orig, wlt_elev_iso, x, y);
wlt_elev_thresh = ones(int_size).*(wlt_elev_int>cont_thresh);
wlt_elev_pos = wlt_elev_int.*(wlt_elev_int>0);


% contour matrix with all the edges of the contoru isolines
cont_mat = contourc(wlt_elev_int, [cont_thresh cont_thresh]);
cont_loc = {};

% Since the matrix is set up in a wierd way I extract the edges of the
% contour line in a matrix.
i = 1;
cont_num = 1;
while i<length(cont_mat(1,:))
    current_cont = zeros(cont_mat(2,i),2);
    current_it = 1;
    for i = i+1:i+cont_mat(2,i)
        current_cont(current_it,:) = ...
            [round(cont_mat(2,i)),round(cont_mat(1,i))];
        current_it = current_it + 1;
    end
    cont_loc{cont_num} = current_cont;
    cont_num = cont_num + 1;
    i = i + 1;
end

tot_cont = zeros(int_size);
for i = 1:length(cont_loc)
    for j = 1:length(cont_loc{i})
        tot_cont(cont_loc{i}(j,1), cont_loc{i}(j,2)) = 1;
    end
end

circles = [];
radii = [];
chosen_cont = zeros(int_size);
for radius = 7:45
    current_cont = zeros(int_size);
    circum_up_thresh = radius*2*pi*1.3;
    circum_low_thresh = radius*2*pi*0.8;
    for i = 1:length(cont_loc)
        cont_circum = 0;
        query = cont_loc{i};
        for j = 1:length(cont_loc{i})-1
            cont_circum = cont_circum + sqrt(...
                (query(j+1,1) - query(j,1))^2 +...
                (query(j+1,2) - query(j,2))^2);
        end
        if cont_circum>circum_up_thresh || cont_circum<circum_low_thresh
            continue
        end
%         mean_x_ind = max(mean(query(:,1))*(256/int_size),1);
%         mean_y_ind = max(mean(query(:,2))*(256/int_size),1);
%         if surf_mean_curv(round(mean_x_ind),round(mean_y_ind)) < 0
%             continue
%         end
        for j = 1:length(cont_loc{i})
            current_cont(cont_loc{i}(j,1), cont_loc{i}(j,2)) = 1;
            chosen_cont(cont_loc{i}(j,1), cont_loc{i}(j,2)) = 1;
        end
    end
    
    
    method = "TwoStage";
    switch method
        case "TwoStage"
            [circ, rad] = imfindcircles(current_cont, [radius-1 radius+5],...
                "Sensitivity", 0.96, "Method", "TwoStage");
            % Sensitivity for cutoff = 0.83
            % Sensitivity for contour points = 0.92
        case "PhaseCode"
            [circ, rad] = imfindcircles(current_cont, [6, 30],...
                "Sensitivity", 0.95, "Method", "PhaseCode");
        otherwise
            circ = [];
            rad = [];
    end
    if ~isempty(circ)
        circles = [circles; circ];
        radii = [radii; rad];
    end
end

% Removing all circles not in an area of positive surface curvature (since
% the mean curvature has to be positive at an attached vortex)
i = 1;
while i < length(circles)
    x_ind = max(round(circles(i,2)*(256/int_size)),1);
    y_ind = max(round(circles(i,1)*(256/int_size)),1);
    if surf_mean_curv(x_ind,y_ind) < 0
        circles(i,:) = [];
        radii(i) = [];
    else
        i = i + 1;
    end
end

% Setting return coordinates and radii for the circles (2*pi x 2*pi grid)
vor_circles = circles.*(2*pi/int_size);
vor_radii = radii.*(2*pi/int_size);


% figure(6), clf
% pcolor(chosen_cont)
% shading flat
% viscircles(circles,radii);


% Preallocating total strength variable
vor_wlt_strength = 0;

% Differential area used in integration
dA = (2*pi/int_size).^2;

% ds = round(max(vor_radii)*2);

for i = 1:length(circles)
    % Side length for integration domain for each vortex
    ds = round(radii(i)*2.5);
    
    % Indeces for circle centers
    x_ind = round(circles(i,2));
    y_ind = round(circles(i,1));
    
    % Integration area indeces
    x_inter = [max(x_ind - ds, 1), min(x_ind + ds, int_size)];
    y_inter = [max(y_ind - ds, 1), min(y_ind + ds, int_size)];
    
    % Preallocating partition of the total area
    par_wlt_elev = wlt_elev_pos(x_inter(1):x_inter(2)...
        ,y_inter(1):y_inter(2));
    for x = 1:length(x_inter)
        for y = 1:length(y_inter)
            if sqrt((x_inter(x) - x_ind)^2 + (y_inter(y) - y_ind)^2) > ds
                par_wlt_elev(x,y) = 0;
            end
        end
    end
    
    % Integrating wavelet transform in the location of the found vortices
    vor_wlt_strength = vor_wlt_strength + dA*sum(par_wlt_elev,'all');
end

end